# Worksheet 1
By <b>Cosmo Elford</b> 201018021

# Task 1, 2 (Getting started)
Getting the LED to turn on with a battery and 220 Ohm resistor. In our week 3 lab session I discovered I didn't have the battery so I used the Raspberry Pi as a power source instead.

*Code:*
```
import RPi.GPIO as GPIO

GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)

GPIO.setup(18,GPIO.OUT)
GPIO.output(18,GPIO.HIGH)
```

<img src="Task1.JPG" width="33.33%">

# Task 3 (3 LEDs)
I added two more LEDs, following the same process as before. Using the GPIO pins, I wrote some code to flash each LED with a `time.sleep(0.1)` (0.1s) delay. I'm also introducing the use of a `while` loop.

*Code:*
```
import RPi.GPIO as GPIO
import time

GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)

# Green
GPIO.setup(24,GPIO.OUT)

# Yellow
GPIO.setup(8,GPIO.OUT)

# Red
GPIO.setup(22,GPIO.OUT)

while True:
    GPIO.output(24,GPIO.HIGH)
    time.sleep(0.1)
    GPIO.output(24,GPIO.LOW)
    time.sleep(0.1)
    GPIO.output(8,GPIO.HIGH)
    time.sleep(0.1)
    GPIO.output(8,GPIO.LOW)
    time.sleep(0.1)
    GPIO.output(22,GPIO.HIGH)
    time.sleep(0.1)
    GPIO.output(22,GPIO.LOW)
    time.sleep(0.1)
```

<img src="Task2.JPG" width="33.33%">

#### Short video:
This shows the sequence between the LEDs

[![Task 3](http://img.youtube.com/vi/7xLNLzCIo9c/0.jpg)](http://www.youtube.com/watch?v=7xLNLzCIo9c)

# Task 4 (Using buttons)

## Adding the first button
Ensure the buttons work and respond with a console output.

<img src="Task4.JPG" width="33.33%">

<img src="Screenshot 06.10.23.png" width="33.33%">


## Adding the second button
Following the same process as before I added a second button using a different GPIO input. I then created a simple program to detect when either button gets pressed.

*Code snippet:*
<br>
```
if (GPIO.input(button_1_pin) == False):
    print("Button 1")
if (GPIO.input(button_2_pin) == False):
    print("Button 2")
```

<img src="Task4_1.JPG" width="33.33%">

## Final bit of Python programming for Task 4
I added 3 blue LEDs, to test they worked correctly I turned them all on using basic GPIO code.

<img src="Task4_2.JPG" width="33.33%">

<br>

**Testing and implementing the code below it follows the spec you have provided. I've added a short video showcasing it in action!**

*Final code:*
```
import RPi.GPIO as GPIO
import time

GPIO.setmode(GPIO.BCM)
GPIO.setwarnings(False)

# LEDs LEFT-TO-RIGHT order
led_pins = [26, 25, 24]

# Button 1, 2
button_1_pin = 22
button_2_pin = 15

GPIO.setup(button_1_pin, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(button_2_pin, GPIO.IN, pull_up_down=GPIO.PUD_UP)

# LEDs
for led_pin in led_pins:
    GPIO.setup(led_pin, GPIO.OUT)

# Initialize variables
sequence_running = True
sequence_direction = 1 # 1 for forward, -1 for reverse
current_led = 0

def update_led_state():
    for i in range(len(led_pins)):
        GPIO.output(led_pins[i], i == current_led)

while True:
    # Button 1
    if (GPIO.input(button_1_pin) == False):
        time.sleep(0.25)
        sequence_running = not sequence_running

        if (sequence_running):
            print("Start")
        else:
            print("Stop")

    # Button 2
    if (GPIO.input(button_2_pin) == False):
        print("Change sequence")
        time.sleep(0.25)
        sequence_direction *= -1
        sequence_running = True

    if sequence_running:
        # Update the LED state
        update_led_state()

        # Wait for 0.5 seconds
        time.sleep(0.5)

        # Move to the next LED
        current_led += sequence_direction

        # Wrap around if needed
        if current_led < 0:
            current_led = len(led_pins) - 1
        elif current_led >= len(led_pins):
            current_led = 0
```

**Short demo:**

[![Task 4_2](http://img.youtube.com/vi/K9MPf20hbis/0.jpg)](http://www.youtube.com/watch?v=K9MPf20hbis)

*Console logs:*

<img src="Screenshot 06.10.23_2.png" width="30%">

# Task 5

### Taking notes...

## Ohm's law:
`V = I * R` to calculate Voltage

```
Where...
V = Voltage
I = Current
R = Resistance
```

*Therefore... if we rearrange Ohms law we can calculate more variables*

`I = V / R` to calculate Current

`R = V / I` to calculate Resistance

### Additionally...

>Ohm’s law states that the voltage (V) across a resistor is proportional to the current (I), where the constant of proportionality is the resistance (R). 

>E.g. If a 200 ohm resistor is attached across the terminals of a 3V battery, then a current of 3 / 200 = 0.015 amps flows through that resistor.

*Futhermore...*

> If there are multiple resistors in series, we can reduce the current by the sum of all the resistors together

`R = R1 + R2 + ... + Rn`

## This can be implemented in Python using 'pint' library

```
import RPi.GPIO as GPIO
from pint import UnitRegistry

ureg = UnitRegistry()

resistance = 5.0 * ureg.ohm
current = 2.5 * ureg.milliampere
voltage = (resistance * current).to('volt')
```

### Moreover, I have created an `ohms_law.py` and `tests.py` in the /ohms_law directory.

Example (testing resistance function):
Expected output from **30V and 3A = 10Ω**

```
def test_resistance_calculation():
    # Test when voltage is 24 V and current is 6 A
    result = calculate_resistance(24, 6)
    expected_result = 4 * ureg.ohm
    assert result == expected_result
    print("Pass")

    # Test when voltage is 30 V and current is 3 A
    result = calculate_resistance(30, 3)
    expected_result = 10 * ureg.ohm
    assert result == expected_result
    print("Pass")
```

```
if __name__ == '__main__':
    # Tests
    test_resistance_calculation()
```

### Other functions have been implemented
*Final code (ohms_law.py):*

```
import pint

ureg = pint.UnitRegistry()

def calculate_voltage(current, resistance):
    # Ensure that the inputs have the correct units (A for current, ohm for resistance)
    current = current * ureg.ampere
    resistance = resistance * ureg.ohm

    voltage = current * resistance
    return voltage

def calculate_current(voltage, resistance):
    # Ensure that the inputs have the correct units (V for voltage, ohm for resistance)
    voltage = voltage * ureg.volt
    resistance = resistance * ureg.ohm

    current = voltage / resistance
    return current

def calculate_resistance(voltage, current):
    # Ensure that the inputs have the correct units (V for voltage, A for current)
    voltage = voltage * ureg.volt
    current = current * ureg.ampere

    resistance = voltage / current
    return resistance

def calculate_serial_resistance(rs):
    # Calculate the total resistance for resistors in series (r_total = R1 + R2 + ... = Rn)
    r_total = sum(rs)
    return r_total
```

*Final code (tests.py):*

```
import pint
from ohms_law import calculate_resistance, calculate_current, calculate_voltage

ureg = pint.UnitRegistry()

def test_voltage_calculation():
    # Test when current is 2 A and resistance is 4 ohms
    result = calculate_voltage(2, 4)
    expected_result = 8 * ureg.volt
    assert result == expected_result
    print("Pass")

    # Test when current is 3 A and resistance is 5 ohms
    result = calculate_voltage(3, 5)
    expected_result = 15 * ureg.volt
    assert result == expected_result
    print("Pass")

    # Add more test cases as needed

def test_current_calculation():
    # Test when voltage is 12 V and resistance is 3 ohms
    result = calculate_current(12, 3)
    expected_result = 4 * ureg.ampere
    assert result == expected_result
    print("Pass")

    # Test when voltage is 15 V and resistance is 5 ohms
    result = calculate_current(15, 5)
    expected_result = 3 * ureg.ampere
    assert result == expected_result
    print("Pass")

    # Add more test cases as needed

def test_resistance_calculation():
    # Test when voltage is 24 V and current is 6 A
    result = calculate_resistance(24, 6)
    expected_result = 4 * ureg.ohm
    assert result == expected_result
    print("Pass")

    # Test when voltage is 30 V and current is 3 A
    result = calculate_resistance(30, 3)
    expected_result = 10 * ureg.ohm
    assert result == expected_result
    print("Pass")

    # Add more test cases as needed

if __name__ == '__main__':
    # Tests
    test_resistance_calculation()
```

*Console logs: *

<img src="Screenshot 08.11.23.png" width="30%">